# cgilib.tcl

proc Cgi_Parse {} {
    global env cgi query
    if {![info exists env(QUERY_STRING)] ||
	    [string length $env(QUERY_STRING)] == 0} {
	if [info exists env(CONTENT_LENGTH)] {
	    set query [read stdin $env(CONTENT_LENGTH)]
	} else {
	    gets stdin query
	}
    } else {
	set query $env(QUERY_STRING)
    }
    regsub -all {\+} $query { } query
    foreach {name value} [split $query &=] {
	if [info exists list($name)] {
	    set cgi($name) [list $cgi($name) [CgiDecode $value]]
	    unset list($name)
	} elseif [info exists cgi($name)] {
	    lappend cgi($name) [CgiDecode $value]
	} else {
	    set cgi($name) [CgiDecode $value]
	    set list($name) 1	;# Need to listify if more values are added
	}
    }
    return [array names cgi]
} 
proc CgiDecode {str} {
    # Protect Tcl special chars
    regsub -all {[][\\\$]} $str {\\&} str
    # Replace %xx sequences with a format command
    regsub -all {%([0-9a-fA-F][0-9a-fA-F])} $str {[format %c 0x\1]} str
    # Replace the format commands with their result
    return [subst $str]
}
proc Cgi_Header {title {bodyparams {}}} {
    puts stdout "\
Content-Type: text/html

<HTML>
<Head>
<title>$title</title>
</Head>
<Body $bodyparams>"
}
proc Cgi_End {} {
    puts </Body>\n</Html>
}
proc Cgi_Redirect {url} {
    puts stdout "\
Content-type: text/html
Location: $url

Please go to $url
"
}
proc H1 {str} {
    Html_Tag H1 {} $str
}
proc H2 {str} {
    Html_Tag H2 {} $str
}
proc H3 {str} {
    Html_Tag H3 {} $str
}
proc H4 {str} {
    Html_Tag H4 {} $str
}
proc H5 {str} {
    Html_Tag H5 {} $str
}
proc H6 {str} {
    Html_Tag H6 {} $str
}
proc P {} {
    puts stdout <p>
}
proc UL {html} {
    puts stdout "<ul>\n$html\n</ul>"
}
proc OL {html} {
    puts stdout "<ol>\n$html\n</ol>"
}
proc LI {html} {
    return "<li>$html"
}
proc Link {text href} {
    return "<a href=\"$href\">$text</a>"
}
proc Html_Tag {tag params str} {
    puts stdout "<[string trim "$tag $params"]>$str</$tag>"
}
proc Form {url {method POST}} {
    puts stdout "<form action=\"$url\" method=$method>"
}
proc Counter {filename} {
    if [catch {open $filename} in] {
	set number 0
    } else {
	set info [read $in]
	close $in
	if ![regexp {[0-9]+} $info number] {
	    return [clock seconds]	;# Bail - race with file access
	}
    }
    incr number
    set out [open $filename.next w]
    puts $out $number
    close $out
    exec mv -f $filename.next $filename
    return $number
}
