/*
 * Example 41-7
 * A canonical Tcl main program and Tcl_AppInit.
 */

/* main.c */
#include <tcl.h>
int Tcl_AppInit(Tcl_Interp *interp);
/*
* Declarations for application-specific command procedures
*/
int RandomCmd(ClientData clientData,
				Tcl_Interp *interp,
				int argc, char *argv[]);
int RandomObjCmd(ClientData clientData,
				Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[]);
int IdentityObjCmd(ClientData clientData,
				Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[]);
int Plus1ObjCmd(ClientData clientData,
				Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[]);

main(int argc, char *argv[]) {
	/*
	 * Initialize your application here.
	 *
	 * Then initialize and run Tcl.
	 */
	Tcl_Main(argc, argv, Tcl_AppInit);
	exit(0);
}
/*
* Tcl_AppInit is called from Tcl_Main
* after the Tcl interpreter has been created,
* and before the script file
* or interactive command loop is entered.
*/
int
Tcl_AppInit(Tcl_Interp *interp) {
	/*
	 * Initialize packages
	 * Tcl_Init sets up the Tcl library facility.
	 */
	if (Tcl_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	/*
	 * Register application-specific commands.
	 */
	Tcl_CreateCommand(interp, "random", RandomCmd,
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	Tcl_CreateObjCommand(interp, "orandom", RandomObjCmd,
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	Tcl_CreateObjCommand(interp, "identity", IdentityObjCmd,
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	Tcl_CreateObjCommand(interp, "plus1", Plus1ObjCmd,
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	/*
	 * Define start-up filename. This file is read in
	 * case the program is run interactively.
	 */
	Tcl_SetVar(interp, "tcl_rcFileName", "~/.mytcl",
		TCL_GLOBAL_ONLY);
	/*
	 * Test of Tcl_Invoke, which is defined on page 538.
	 */
	Tcl_Invoke(interp, "set", "foo", "$xyz [foo] {", NULL);
	return TCL_OK;
}


